import {
  ChangeDetectionStrategy,
  Component,
  Input,
} from "@angular/core";
import {SharedAbstractPresentational} from "@app/shared/components/presentationals/shared-abstract/shared-abstract.presentational";
import {PackageStatusEnum} from "@shared/enums/business/package-status.enum";
import {
  AppRoutesEnum,
  PreservationPlanningRoutesEnum,
} from "@shared/enums/routes.enum";
import {AipCopyList} from "@shared/models/business/aip-copy-list.model";
import {AipCopy} from "@shared/models/business/aip-copy.model";
import {
  isNonEmptyArray,
  isNullOrUndefined,
  TRANSLATE,
  urlSeparator,
} from "solidify-frontend";

@Component({
  selector: "dlcm-aip-status-summary",
  templateUrl: "./aip-status-summary.presentational.html",
  styleUrls: ["./aip-status-summary.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class AipStatusSummaryPresentational extends SharedAbstractPresentational {
  @Input()
  aipCopyList: AipCopyList;

  private readonly PATH_TO_AIP_STORAGION: string = "#" + urlSeparator + AppRoutesEnum.preservation + urlSeparator + PreservationPlanningRoutesEnum.aip + urlSeparator;

  get packageStatusEnum(): typeof PackageStatusEnum {
    return PackageStatusEnum;
  }

  getStatusLabel(status: PackageStatusEnum): string {
    if (isNullOrUndefined(status)) {
      return TRANSLATE("COPY_MISSING");
    }
    if (status === PackageStatusEnum.IN_PREPARATION) {
      return TRANSLATE("IN_PREPARATION");
    }
    if (status === PackageStatusEnum.REINDEXING) {
      return TRANSLATE("REINDEXING");
    }
    return status;
  }

  getStoragionNumber(aipCopy: AipCopy): number {
    const urlSegment = aipCopy.storageUrl.split(urlSeparator);
    const storagion = urlSegment[urlSegment.length - 2];
    const numberExtracted = storagion.match(/\d+/g);
    const number = isNonEmptyArray(numberExtracted) ? +numberExtracted[0] : 1;
    return number;
  }

  getUrl(aipCopy: AipCopy): string {
    const url = this.PATH_TO_AIP_STORAGION + this.getStoragionNumber(aipCopy);
    if (isNullOrUndefined(aipCopy.status)) {
     return url;
    }
    return url + urlSeparator + PreservationPlanningRoutesEnum.aipDetail + urlSeparator + aipCopy.aip.resId;
  }
}
