import {
  ChangeDetectionStrategy,
  Component,
  Host,
  Inject,
  NgZone,
} from "@angular/core";
import {
  MAT_SNACK_BAR_DATA,
  MatSnackBarRef,
} from "@angular/material/snack-bar";
import {
  AbstractSnackbarPresentational,
  isNullOrUndefined,
  NotificationTypeEnum,
  SnackbarData,
} from "solidify-frontend";

@Component({
  selector: "dlcm-shared-snackbar",
  templateUrl: "./shared-snackbar.presentational.html",
  styleUrls: ["./shared-snackbar.presentational.scss"],
  changeDetection: ChangeDetectionStrategy.OnPush,
})
export class SharedSnackbarPresentational extends AbstractSnackbarPresentational {
  get notificationTypeEnum(): typeof NotificationTypeEnum {
    return NotificationTypeEnum;
  }

  constructor(@Host() public snackbarRef: MatSnackBarRef<SharedSnackbarPresentational>,
              @Inject(MAT_SNACK_BAR_DATA) public data: SharedSnackbarOption,
              private _ngZone: NgZone) {
    super();
  }

  getIcon(): string | undefined {
    if (!isNullOrUndefined(this.data.icon)) {
      return this.data.icon;
    }

    switch (this.data.category) {
      case NotificationTypeEnum.success:
        return "check_circle";
      case NotificationTypeEnum.information:
        return "info";
      case NotificationTypeEnum.warning:
        return "warning";
      case NotificationTypeEnum.error:
        return "error";
    }

    return undefined;
  }

  close($event: MouseEvent): void {
    this._ngZone.run(() => this.snackbarRef.dismiss());
  }
}

export interface SharedSnackbarOption extends SnackbarData {
  icon?: string;
}
